@@ -69,6 +69,8 @@ module System.Directory.Tree (
        , sortDir
        , sortDirShape
        , filterDir
+       -- *** Low-level
+       , transformDir
        -- ** Navigation
        , dropTo
        -- ** Operators
@@ -248,9 +250,7 @@ readDirectory = readDirectoryWith readFile
 -- | same as readDirectory but allows us to, for example, use 
 -- ByteString.readFile to return a tree of ByteStrings.
 readDirectoryWith :: (FilePath -> IO a) -> FilePath -> IO (AnchoredDirTree a)
-readDirectoryWith f p = do (b:/t) <- buildWith' buildAtOnce' f p
-                           let t' = removeNonexistent t
-                           return ( b:/t') 
+readDirectoryWith f p = buildWith' buildAtOnce' f p
 
 
 -- | A "lazy" version of `readDirectoryWith` that does IO operations as needed
@@ -258,9 +258,7 @@ readDirectoryWith f p = do (b:/t) <- buildWith' buildAtOnce' f p
 -- /NOTE:/ This function uses unsafePerformIO under the hood. I believe our use
 -- here is safe, but this function is experimental in this release:
 readDirectoryWithL :: (FilePath -> IO a) -> FilePath -> IO (AnchoredDirTree a)
-readDirectoryWithL f p = do (b:/t) <- buildWith' buildLazilyUnsafe' f p
-                            let t' = removeNonexistent t
-                            return ( b:/t') 
+readDirectoryWithL f p = buildWith' buildLazilyUnsafe' f p
 
 
 -- | write a DirTree of strings to disk. Clobbers files of the same name. 
@@ -284,7 +282,6 @@ writeDirectoryWith f (b:/t) = (b:/) <$> write' b t
               do let bas = b'</>n
                  createDirectoryIfMissing True bas
                  Dir n <$> mapM (write' bas) cs
-           -- INTERESTING: have to rebuild Failed constr. to get to typecheck:
           write' _ (Failed n e) = return $ Failed n e
 
 
@@ -391,7 +388,7 @@ failures = filter failed . flattenDir
 
 -- | maps a function to convert Failed DirTrees to Files or Dirs
 failedMap :: (FileName -> IOException -> DirTree a) -> DirTree a -> DirTree a
-failedMap f = transform unFail
+failedMap f = transformDir unFail
     where unFail (Failed n e) = f n e
           unFail c            = c
                           
@@ -410,7 +407,7 @@ sortDirShape = sortDirBy comparingShape  where
 
   -- HELPER:
 sortDirBy :: (DirTree a -> DirTree a -> Ordering) -> DirTree a -> DirTree a
-sortDirBy cf = transform sortD
+sortDirBy cf = transformDir sortD
     where sortD (Dir n cs) = Dir n (sortBy cf cs)
           sortD c          = c
 
@@ -477,7 +474,7 @@ dropTo _ _ = Nothing
 -- its children, of course) when the predicate returns False. The topmost 
 -- constructor will always be preserved:
 filterDir :: (DirTree a -> Bool) -> DirTree a -> DirTree a
-filterDir p = transform filterD
+filterDir p = transformDir filterD
     where filterD (Dir n cs) = Dir n $ filter p cs
           filterD c          = c
 
@@ -582,17 +579,17 @@ removeNonexistent = filterDir isOkConstructor
            isOkError = not . isDoesNotExistErrorType . ioeGetErrorType . err
 
 
----- THIS COULD BE USEFUL TO EXPORT:
-
--- at Dir constructor, apply transformation function to all of directory's
--- contents, then remove the Nothing's and recurse.
--- ALWAYS PRESERVES TOPMOST CONSTRUCTOR:
-transform :: (DirTree a -> DirTree a) -> DirTree a -> DirTree a
-transform f t = case f t of
-                     (Dir n cs) -> Dir n $ map (transform f) cs
+-- | At 'Dir' constructor, apply transformation function to all of directory's
+-- contents, then remove the Nothing's and recurse. This always preserves the
+-- topomst constructor.
+transformDir :: (DirTree a -> DirTree a) -> DirTree a -> DirTree a
+transformDir f t = case f t of
+                     (Dir n cs) -> Dir n $ map (transformDir f) cs
                      t'         -> t'
 
 -- Lenses, generated with TH from "lens" -----------
+-- TODO deprecate these? Pain in the ass to generate, and maybe it's intended
+--      for users to generate their own lenses.
 _contents :: 
             Applicative f =>
             ([DirTree a] -> f [DirTree a]) -> DirTree a -> f (DirTree a)
